/*

*/

import serverFramework.*;

import java.time.format.DateTimeFormatter;
import java.time.OffsetDateTime;
import java.time.ZoneOffset;

import java.util.Scanner;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileInputStream;
import java.io.IOException;

/**
   This is an example of how you can use the HttpApplicationServer
   framework in the server package to implement a very simple
   microservice api.

   The microservice api implemented here takes an integer
   parameter between 1 and 25,000 and returns a photo URL
   pointing at the Unslpash.com servers. For example
      http://localhost:9090/?n=61
   will return, in the response entity body, the URL
   for photo number 61.
*/
public class Microservice extends HttpMethodsAdaptor
{
   /**
      Give this microservice server an implementation of GET.

      This GET method expects a single integer URL query
      parameter, n, that is between 1 and 25,000, and
      returns the n'th photo URL from the file photo-urls.txt.
   */
   @Override
   public void doGet(HttpRequest req, HttpResponse res)
   {
      if ( ! req.resourceName.equals("/photoURL") )
      {
         sendErrorResponse(req, res,
             "400", "Bad Request",
             "This microservice only supplies photo URLs.");
         return;  // Finished with this request.
      }

      System.out.println("SERVER: Hw4 photo microservice:");

      // Get the URL's query string from the req object.
      final String query = req.queryString;
      if (null == query)
      {
         sendErrorResponse(req, res,
             "400", "Bad Request",
             "Null query string.");
         return;  // Finished with this request.
      }
      // Parse it.
      final int i1 = query.indexOf("=");
      String p1 = query.substring(i1 + 1);
      int n;
      try
      {
         n = Integer.parseInt(p1);
      }
      catch (NumberFormatException e)
      {
         sendErrorResponse(req, res,
             "400", "Bad Request",
             p1 + " does not parse to an int.");
         return;  // Finished with this request.
      }
      if (n < 1 || n > 25_000)
      {
         sendErrorResponse(req, res,
             "400", "Bad Request",
             "n = " + n + " is out of range.");
         return;  // Finished with this request.
      }

      System.out.println("=====> Parsed query parameter to: " + n);

      // Read line n from the local file photo-urls.txt.
      String photoURL = null;
      final File file = new File("microservice/photo-urls.txt");
      try (final Scanner inFile = new Scanner(file))
      {
         for (int i = 0; i < n; ++i)
         {
             photoURL = inFile.nextLine();
         }
      }
      catch (FileNotFoundException e)
      {
         System.out.println("SERVER: Cannot find file: photo-urls.txt");
         System.exit(-1);
      }
      System.out.println("=====> Found URL: " + photoURL);

      // Build the HTML response page.
      final String body = photoURL;

      // Add the response line.
      res.setResponseLine("HTTP/1.1 200 OK");

      // Add the response headers.
      final String date = OffsetDateTime.now(ZoneOffset.UTC)
                            .format(DateTimeFormatter.RFC_1123_DATE_TIME);
      res.addHeader("Date: " + date);
      res.addHeader("Server: Microservice.java");
      res.addHeader("Connection: close");
      res.addHeader("Content-type: text/html");
      res.addHeader("Content-Length: " + body.length());
      res.addHeader("Access-Control-Allow-Origin: *");

      // Add the entity body.
      res.setEntityBody( body.getBytes() );

      return;
   }


   /**
      The main() method configures the server using
      the httpserver.properties file and command-line
      arguments.
   */
   public static void main(String[] args)
   {
      String serverRoot = "microservice"; // default value
      int portNumber = 9090;              // default value

      if (0 < args.length) portNumber = Integer.parseInt(args[0]);
      if (1 < args.length) serverRoot = args[1];

      // Instantiate and start up an instance of the server.
      new HttpApplicationServer(portNumber,
                                serverRoot,
                                new Microservice()); // HTTP methods
   }
}
